﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using C1.Silverlight.Pdf;
using C1.Silverlight.FlexGrid;

namespace C1.Silverlight.FlexGrid.PrintPreview
{
    public static class C1FlexGridPdfExtension
    {
        public static C1PdfDocument ToPdf(this C1FlexGrid grid)
        {
            return grid.ToPdf(new PageSettings() { 
                Margins = new Thickness(72), 
                ScaleMode = ScaleMode.PageWidth,
                IsLandscape = false, 
                PaperKind = PaperKind.Letter, 
                FooterLeftText = "[Date], [Time]",
                FooterRightText = "Page [Page] of [PageCount]",
                ShowHeaderSeparator = false,
                ShowFooterSeparator = true });
        }

        //public static C1PdfDocument ToPdf(this C1FlexGrid grid, Thickness pageMargins, ScaleMode scaleMode, bool isLandscape, PaperKind paperKind, Size customPageSize)
        public static C1PdfDocument ToPdf(this C1FlexGrid grid, PageSettings pageSettings)
        {
            // get root element to lay out the PDF pages
            Panel root = null;
            for (var parent = grid.Parent as FrameworkElement; parent != null; parent = parent.Parent as FrameworkElement)
            {
                if (parent is Panel)
                {
                    root = parent as Panel;
                }
            }
            
            // create pdf document
            var pdf = new C1PdfDocument();

            // set landscape
            pdf.Landscape = pageSettings.IsLandscape;

            // set paper kind
            pdf.PaperKind = pageSettings.PaperKind;

            // set custom page size
            if (pageSettings.PaperKind == PaperKind.Custom)
            {
                pdf.PageSize = pageSettings.CustomPageSize;
            }

            // get page size
            var rc = pdf.PageRectangle;

            // create panel to hold elements while they render
            var pageTemplate = new PageTemplate();
            pageTemplate.Width = rc.Width;
            pageTemplate.Height = rc.Height;
            pageTemplate.SetPageMargin(pageSettings.Margins);
            root.Children.Add(pageTemplate);

            // render grid into PDF document
            var sz = new Size(rc.Width - pageSettings.Margins.Left - pageSettings.Margins.Right, rc.Height - pageSettings.Margins.Top - pageSettings.Margins.Bottom);
            var pages = grid.GetPageImages(pageSettings.ScaleMode, sz, 100);
            for (int i = 0; i < pages.Count; i++)
            {
                // skip a page when necessary
                if (i > 0)
                {
                    pdf.NewPage();
                }

                // set content
                pageTemplate.PageContent.Child = pages[i];
                pageTemplate.PageContent.Stretch = System.Windows.Media.Stretch.Uniform;

                // set header/footer text
                pageTemplate.HeaderLeft.Text = FormatPlaceHolders(pageSettings.HeaderLeftText, i + 1, pages.Count);
                pageTemplate.HeaderRight.Text = FormatPlaceHolders(pageSettings.HeaderRightText, i + 1, pages.Count);
                pageTemplate.FooterLeft.Text = FormatPlaceHolders(pageSettings.FooterLeftText, i + 1, pages.Count);
                pageTemplate.FooterRight.Text = FormatPlaceHolders(pageSettings.FooterRightText, i + 1, pages.Count);

                // set header separator
                if (pageSettings.ShowHeaderSeparator)
                {
                    pageTemplate.BorderHeader.BorderThickness = new Thickness(0, 0, 0, 1);
                }
                else
                {
                    pageTemplate.BorderHeader.BorderThickness = new Thickness(0, 0, 0, 0);
                }

                // set footer separator
                if (pageSettings.ShowFooterSeparator)
                {
                    pageTemplate.BorderFooter.BorderThickness = new Thickness(0, 1, 0, 0);
                }
                else
                {
                    pageTemplate.BorderFooter.BorderThickness = new Thickness(0, 0, 0, 0);
                }

                // measure page element
                pageTemplate.Measure(new Size(rc.Width, rc.Height));
                pageTemplate.UpdateLayout();

                // add page element to PDF
                pdf.DrawElement(pageTemplate, rc);
            }

            // done with template
            root.Children.Remove(pageTemplate);

            return pdf;
        }

        public static string FormatPlaceHolders(string text, int page, int pageCount)
        {
            if (text == null)
                return text; // sometimes it's null

            text = text.Replace("[Date]", DateTime.Today.ToShortDateString());
            text = text.Replace("[Time]", DateTime.Now.ToShortTimeString());
            text = text.Replace("[Page]", page.ToString());
            text = text.Replace("[PageCount]", pageCount.ToString());
            return text;
        }
    }
}
